@@ -67,10 +67,10 @@ template<typename... Ts> class TemplatableKeyValuePair {
 // the callback is invoked synchronously while the message is on the stack).
 class ActionResponse {
  public:
-  ActionResponse(bool success, const std::string &error_message) : success_(success), error_message_(error_message) {}
+  ActionResponse(bool success, StringRef error_message) : success_(success), error_message_(error_message) {}
 
 #ifdef USE_API_HOMEASSISTANT_ACTION_RESPONSES_JSON
-  ActionResponse(bool success, const std::string &error_message, const uint8_t *data, size_t data_len)
+  ActionResponse(bool success, StringRef error_message, const uint8_t *data, size_t data_len)
       : success_(success), error_message_(error_message) {
     if (data == nullptr || data_len == 0)
       return;
@@ -147,13 +147,23 @@ template<typename... Ts> class HomeAssistantServiceCallAction : public Action<Ts
   void play(const Ts &...x) override {
     HomeassistantActionRequest resp;
     std::string service_value = this->service_.value(x...);
-    resp.set_service(StringRef(service_value));
+    resp.service = StringRef(service_value);
     resp.is_event = this->flags_.is_event;
-    this->populate_service_map(resp.data, this->data_, x...);
-    this->populate_service_map(resp.data_template, this->data_template_, x...);
-    this->populate_service_map(resp.variables, this->variables_, x...);
+
+    // Local storage for lambda-evaluated strings - lives until after send
+    FixedVector<std::string> data_storage;
+    FixedVector<std::string> data_template_storage;
+    FixedVector<std::string> variables_storage;
+
+    this->populate_service_map(resp.data, this->data_, data_storage, x...);
+    this->populate_service_map(resp.data_template, this->data_template_, data_template_storage, x...);
+    this->populate_service_map(resp.variables, this->variables_, variables_storage, x...);
 
 #ifdef USE_API_HOMEASSISTANT_ACTION_RESPONSES
+#ifdef USE_API_HOMEASSISTANT_ACTION_RESPONSES_JSON
+    // IMPORTANT: Declare at outer scope so it lives until send_homeassistant_action returns.
+    std::string response_template_value;
+#endif
     if (this->flags_.wants_status) {
       // Generate a unique call ID for this service call
       static uint32_t call_id_counter = 1;
@@ -164,8 +174,8 @@ template<typename... Ts> class HomeAssistantServiceCallAction : public Action<Ts
         resp.wants_response = true;
         // Set response template if provided
         if (this->flags_.has_response_template) {
-          std::string response_template_value = this->response_template_.value(x...);
-          resp.response_template = response_template_value;
+          response_template_value = this->response_template_.value(x...);
+          resp.response_template = StringRef(response_template_value);
         }
       }
 #endif
@@ -205,12 +215,31 @@ template<typename... Ts> class HomeAssistantServiceCallAction : public Action<Ts
   }
 
   template<typename VectorType, typename SourceType>
-  static void populate_service_map(VectorType &dest, SourceType &source, Ts... x) {
+  static void populate_service_map(VectorType &dest, SourceType &source, FixedVector<std::string> &value_storage,
+                                   Ts... x) {
     dest.init(source.size());
+
+    // Count non-static strings to allocate exact storage needed
+    size_t lambda_count = 0;
+    for (const auto &it : source) {
+      if (!it.value.is_static_string()) {
+        lambda_count++;
+      }
+    }
+    value_storage.init(lambda_count);
+
     for (auto &it : source) {
       auto &kv = dest.emplace_back();
-      kv.set_key(StringRef(it.key));
-      kv.value = it.value.value(x...);
+      kv.key = StringRef(it.key);
+
+      if (it.value.is_static_string()) {
+        // Static string from YAML - zero allocation
+        kv.value = StringRef(it.value.get_static_string());
+      } else {
+        // Lambda evaluation - store result, reference it
+        value_storage.push_back(it.value.value(x...));
+        kv.value = StringRef(value_storage.back());
+      }
     }
   }
 
