@@ -1,4 +1,5 @@
 import esphome.codegen as cg
+from esphome.components.logger import request_log_listener
 from esphome.components.zephyr import zephyr_add_prj_conf
 import esphome.config_validation as cv
 from esphome.const import CONF_ID, CONF_LOGS, CONF_TYPE
@@ -25,5 +26,8 @@ CONFIG_SCHEMA = cv.All(
 async def to_code(config):
     var = cg.new_Pvariable(config[CONF_ID])
     zephyr_add_prj_conf("BT_NUS", True)
-    cg.add(var.set_expose_log(config[CONF_TYPE] == CONF_LOGS))
+    expose_log = config[CONF_TYPE] == CONF_LOGS
+    cg.add(var.set_expose_log(expose_log))
+    if expose_log:
+        request_log_listener()  # Request a log listener slot for BLE NUS log streaming
     await cg.register_component(var, config)
