@@ -1,12 +1,10 @@
 #pragma once
 
-#include <string>
-
 namespace esphome {
 namespace dashboard_import {
 
-const std::string &get_package_import_url();
-void set_package_import_url(std::string url);
+const char *get_package_import_url();
+void set_package_import_url(const char *url);
 
 }  // namespace dashboard_import
 }  // namespace esphome
