@@ -23,11 +23,18 @@ from esphome.helpers import copy_file_if_changed
 from .boards import BOARDS, ESP8266_LD_SCRIPTS
 from .const import (
     CONF_EARLY_PIN_INIT,
+    CONF_ENABLE_SERIAL,
+    CONF_ENABLE_SERIAL1,
     CONF_RESTORE_FROM_FLASH,
     KEY_BOARD,
     KEY_ESP8266,
     KEY_FLASH_SIZE,
     KEY_PIN_INITIAL_STATES,
+    KEY_SERIAL1_REQUIRED,
+    KEY_SERIAL_REQUIRED,
+    KEY_WAVEFORM_REQUIRED,
+    enable_serial,
+    enable_serial1,
     esp8266_ns,
 )
 from .gpio import PinInitialState, add_pin_initial_states_array
@@ -170,6 +177,8 @@ CONFIG_SCHEMA = cv.All(
             cv.Optional(CONF_BOARD_FLASH_MODE, default="dout"): cv.one_of(
                 *BUILD_FLASH_MODES, lower=True
             ),
+            cv.Optional(CONF_ENABLE_SERIAL): cv.boolean,
+            cv.Optional(CONF_ENABLE_SERIAL1): cv.boolean,
         }
     ),
     set_core_data,
@@ -191,7 +200,13 @@ async def to_code(config):
     cg.add_define(ThreadModel.SINGLE)
 
     cg.add_platformio_option(
-        "extra_scripts", ["pre:testing_mode.py", "post:post_build.py"]
+        "extra_scripts",
+        [
+            "pre:testing_mode.py",
+            "pre:exclude_updater.py",
+            "pre:exclude_waveform.py",
+            "post:post_build.py",
+        ],
     )
 
     conf = config[CONF_FRAMEWORK]
@@ -224,6 +239,12 @@ async def to_code(config):
     if config[CONF_EARLY_PIN_INIT]:
         cg.add_define("USE_ESP8266_EARLY_PIN_INIT")
 
+    # Allow users to force-enable Serial objects for use in lambdas or external libraries
+    if config.get(CONF_ENABLE_SERIAL):
+        enable_serial()
+    if config.get(CONF_ENABLE_SERIAL1):
+        enable_serial1()
+
     # Arduino 2 has a non-standards conformant new that returns a nullptr instead of failing when
     # out of memory and exceptions are disabled. Since Arduino 2.6.0, this flag can be used to make
     # new abort instead. Use it so that OOM fails early (on allocation) instead of on dereference of
@@ -263,10 +284,43 @@ async def to_code(config):
             cg.add_platformio_option("board_build.ldscript", ld_script)
 
     CORE.add_job(add_pin_initial_states_array)
+    CORE.add_job(finalize_waveform_config)
+    CORE.add_job(finalize_serial_config)
+
+
+@coroutine_with_priority(CoroPriority.WORKAROUNDS)
+async def finalize_waveform_config() -> None:
+    """Add waveform stubs define if waveform is not required.
+
+    This runs at WORKAROUNDS priority (-999) to ensure all components
+    have had a chance to call require_waveform() first.
+    """
+    if not CORE.data.get(KEY_ESP8266, {}).get(KEY_WAVEFORM_REQUIRED, False):
+        # No component needs waveform - enable stubs and exclude Arduino waveform code
+        # Use build flag (visible to both C++ code and PlatformIO script)
+        cg.add_build_flag("-DUSE_ESP8266_WAVEFORM_STUBS")
+
+
+@coroutine_with_priority(CoroPriority.WORKAROUNDS)
+async def finalize_serial_config() -> None:
+    """Exclude unused Arduino Serial objects from the build.
+
+    This runs at WORKAROUNDS priority (-999) to ensure all components
+    have had a chance to call enable_serial() or enable_serial1() first.
+
+    The Arduino ESP8266 core defines two global Serial objects (32 bytes each).
+    By adding NO_GLOBAL_SERIAL or NO_GLOBAL_SERIAL1 build flags, we prevent
+    unused Serial objects from being linked, saving 32 bytes each.
+    """
+    esp8266_data = CORE.data.get(KEY_ESP8266, {})
+    if not esp8266_data.get(KEY_SERIAL_REQUIRED, False):
+        cg.add_build_flag("-DNO_GLOBAL_SERIAL")
+    if not esp8266_data.get(KEY_SERIAL1_REQUIRED, False):
+        cg.add_build_flag("-DNO_GLOBAL_SERIAL1")
 
 
 # Called by writer.py
-def copy_files():
+def copy_files() -> None:
     dir = Path(__file__).parent
     post_build_file = dir / "post_build.py.script"
     copy_file_if_changed(
@@ -278,3 +332,13 @@ def copy_files():
         testing_mode_file,
         CORE.relative_build_path("testing_mode.py"),
     )
+    exclude_updater_file = dir / "exclude_updater.py.script"
+    copy_file_if_changed(
+        exclude_updater_file,
+        CORE.relative_build_path("exclude_updater.py"),
+    )
+    exclude_waveform_file = dir / "exclude_waveform.py.script"
+    copy_file_if_changed(
+        exclude_waveform_file,
+        CORE.relative_build_path("exclude_waveform.py"),
+    )
