@@ -1,7 +1,6 @@
 #include "zone_coordinate_number.h"
 
-namespace esphome {
-namespace ld2450 {
+namespace esphome::ld2450 {
 
 ZoneCoordinateNumber::ZoneCoordinateNumber(uint8_t zone) : zone_(zone) {}
 
@@ -10,5 +9,4 @@ void ZoneCoordinateNumber::control(float value) {
   this->parent_->set_zone_coordinate(this->zone_);
 }
 
-}  // namespace ld2450
-}  // namespace esphome
+}  // namespace esphome::ld2450
