@@ -12,41 +12,77 @@ namespace esphome::logger {
 
 static const char *const TAG = "logger";
 
-#ifdef USE_ESP32
-// Implementation for ESP32 (multi-task platform with task-specific tracking)
-// Main task always uses direct buffer access for console output and callbacks
+#if defined(USE_ESP32) || defined(USE_HOST) || defined(USE_LIBRETINY)
+// Implementation for multi-threaded platforms (ESP32 with FreeRTOS, Host with pthreads, LibreTiny with FreeRTOS)
+// Main thread/task always uses direct buffer access for console output and callbacks
 //
-// For non-main tasks:
+// For non-main threads/tasks:
 //  - WITH task log buffer: Prefer sending to ring buffer for async processing
 //    - Avoids allocating stack memory for console output in normal operation
-//    - Prevents console corruption from concurrent writes by multiple tasks
+//    - Prevents console corruption from concurrent writes by multiple threads
 //    - Messages are serialized through main loop for proper console output
 //    - Fallback to emergency console logging only if ring buffer is full
 //  - WITHOUT task log buffer: Only emergency console output, no callbacks
+//
+// Optimized for the common case: 99.9% of logs come from the main thread
 void HOT Logger::log_vprintf_(uint8_t level, const char *tag, int line, const char *format, va_list args) {  // NOLINT
   if (level > this->level_for(tag))
     return;
 
+#if defined(USE_ESP32) || defined(USE_LIBRETINY)
+  // Get task handle once - used for both main task check and passing to non-main thread handler
   TaskHandle_t current_task = xTaskGetCurrentTaskHandle();
-  bool is_main_task = (current_task == main_task_);
+  const bool is_main_task = (current_task == this->main_task_);
+#else  // USE_HOST
+  const bool is_main_task = pthread_equal(pthread_self(), this->main_thread_);
+#endif
 
-  // Check and set recursion guard - uses pthread TLS for per-task state
-  if (this->check_and_set_task_log_recursion_(is_main_task)) {
-    return;  // Recursion detected
-  }
-
-  // Main task uses the shared buffer for efficiency
-  if (is_main_task) {
+  // Fast path: main thread, no recursion (99.9% of all logs)
+  if (is_main_task && !this->main_task_recursion_guard_) [[likely]] {
+    RecursionGuard guard(this->main_task_recursion_guard_);
+    // Format and send to both console and callbacks
     this->log_message_to_buffer_and_send_(level, tag, line, format, args);
-    this->reset_task_log_recursion_(is_main_task);
     return;
   }
 
+  // Main task with recursion - silently drop to prevent infinite loop
+  if (is_main_task) {
+    return;
+  }
+
+  // Non-main thread handling (~0.1% of logs)
+#if defined(USE_ESP32) || defined(USE_LIBRETINY)
+  this->log_vprintf_non_main_thread_(level, tag, line, format, args, current_task);
+#else  // USE_HOST
+  this->log_vprintf_non_main_thread_(level, tag, line, format, args);
+#endif
+}
+
+// Handles non-main thread logging only
+// Kept separate from hot path to improve instruction cache performance
+#if defined(USE_ESP32) || defined(USE_LIBRETINY)
+void Logger::log_vprintf_non_main_thread_(uint8_t level, const char *tag, int line, const char *format, va_list args,
+                                          TaskHandle_t current_task) {
+#else  // USE_HOST
+void Logger::log_vprintf_non_main_thread_(uint8_t level, const char *tag, int line, const char *format, va_list args) {
+#endif
+  // Check if already in recursion for this non-main thread/task
+  if (this->is_non_main_task_recursive_()) {
+    return;
+  }
+
+  // RAII guard - automatically resets on any return path
+  auto guard = this->make_non_main_task_guard_();
+
   bool message_sent = false;
 #ifdef USE_ESPHOME_TASK_LOG_BUFFER
-  // For non-main tasks, queue the message for callbacks - but only if we have any callbacks registered
+  // For non-main threads/tasks, queue the message for callbacks
+#if defined(USE_ESP32) || defined(USE_LIBRETINY)
   message_sent =
       this->log_buffer_->send_message_thread_safe(level, tag, static_cast<uint16_t>(line), current_task, format, args);
+#else  // USE_HOST
+  message_sent = this->log_buffer_->send_message_thread_safe(level, tag, static_cast<uint16_t>(line), format, args);
+#endif
   if (message_sent) {
     // Enable logger loop to process the buffered message
     // This is safe to call from any context including ISRs
@@ -54,39 +90,41 @@ void HOT Logger::log_vprintf_(uint8_t level, const char *tag, int line, const ch
   }
 #endif  // USE_ESPHOME_TASK_LOG_BUFFER
 
-  // Emergency console logging for non-main tasks when ring buffer is full or disabled
+  // Emergency console logging for non-main threads when ring buffer is full or disabled
   // This is a fallback mechanism to ensure critical log messages are visible
-  // Note: This may cause interleaved/corrupted console output if multiple tasks
+  // Note: This may cause interleaved/corrupted console output if multiple threads
   // log simultaneously, but it's better than losing important messages entirely
+#ifdef USE_HOST
+  if (!message_sent) {
+    // Host always has console output - no baud_rate check needed
+    static const size_t MAX_CONSOLE_LOG_MSG_SIZE = 512;
+#else
   if (!message_sent && this->baud_rate_ > 0) {  // If logging is enabled, write to console
     // Maximum size for console log messages (includes null terminator)
     static const size_t MAX_CONSOLE_LOG_MSG_SIZE = 144;
+#endif
     char console_buffer[MAX_CONSOLE_LOG_MSG_SIZE];  // MUST be stack allocated for thread safety
     uint16_t buffer_at = 0;                         // Initialize buffer position
     this->format_log_to_buffer_with_terminator_(level, tag, line, format, args, console_buffer, &buffer_at,
                                                 MAX_CONSOLE_LOG_MSG_SIZE);
-    // Add newline if platform needs it (ESP32 doesn't add via write_msg_)
-    this->add_newline_to_buffer_if_needed_(console_buffer, &buffer_at, MAX_CONSOLE_LOG_MSG_SIZE);
+    // Add newline before writing to console
+    this->add_newline_to_buffer_(console_buffer, &buffer_at, MAX_CONSOLE_LOG_MSG_SIZE);
     this->write_msg_(console_buffer, buffer_at);
   }
 
-  // Reset the recursion guard for this task
-  this->reset_task_log_recursion_(is_main_task);
+  // RAII guard automatically resets on return
 }
 #else
-// Implementation for all other platforms
+// Implementation for all other platforms (single-task, no threading)
 void HOT Logger::log_vprintf_(uint8_t level, const char *tag, int line, const char *format, va_list args) {  // NOLINT
   if (level > this->level_for(tag) || global_recursion_guard_)
     return;
 
-  global_recursion_guard_ = true;
-
+  RecursionGuard guard(global_recursion_guard_);
   // Format and send to both console and callbacks
   this->log_message_to_buffer_and_send_(level, tag, line, format, args);
-
-  global_recursion_guard_ = false;
 }
-#endif  // !USE_ESP32
+#endif  // USE_ESP32 / USE_HOST / USE_LIBRETINY
 
 #ifdef USE_STORE_LOG_STR_IN_FLASH
 // Implementation for ESP8266 with flash string support.
@@ -115,7 +153,7 @@ void Logger::log_vprintf_(uint8_t level, const char *tag, int line, const __Flas
   if (level > this->level_for(tag) || global_recursion_guard_)
     return;
 
-  global_recursion_guard_ = true;
+  RecursionGuard guard(global_recursion_guard_);
   this->tx_buffer_at_ = 0;
 
   // Copy format string from progmem
@@ -125,9 +163,8 @@ void Logger::log_vprintf_(uint8_t level, const char *tag, int line, const __Flas
     this->tx_buffer_[this->tx_buffer_at_++] = ch = (char) progmem_read_byte(format_pgm_p++);
   }
 
-  // Buffer full from copying format
+  // Buffer full from copying format - RAII guard handles cleanup on return
   if (this->tx_buffer_at_ >= this->tx_buffer_size_) {
-    global_recursion_guard_ = false;  // Make sure to reset the recursion guard before returning
     return;
   }
 
@@ -141,13 +178,13 @@ void Logger::log_vprintf_(uint8_t level, const char *tag, int line, const __Flas
       this->tx_buffer_at_ - msg_start;  // Don't subtract 1 - tx_buffer_at_ is already at the null terminator position
 
   // Listeners get message first (before console write)
+#ifdef USE_LOG_LISTENERS
   for (auto *listener : this->log_listeners_)
     listener->on_log(level, tag, this->tx_buffer_ + msg_start, msg_length);
+#endif
 
   // Write to console starting at the msg_start
   this->write_tx_buffer_to_console_(msg_start, &msg_length);
-
-  global_recursion_guard_ = false;
 }
 #endif  // USE_STORE_LOG_STR_IN_FLASH
 
@@ -167,15 +204,26 @@ Logger::Logger(uint32_t baud_rate, size_t tx_buffer_size) : baud_rate_(baud_rate
   this->main_task_ = xTaskGetCurrentTaskHandle();
 #elif defined(USE_ZEPHYR)
   this->main_task_ = k_current_get();
+#elif defined(USE_HOST)
+  this->main_thread_ = pthread_self();
 #endif
 }
 #ifdef USE_ESPHOME_TASK_LOG_BUFFER
 void Logger::init_log_buffer(size_t total_buffer_size) {
+#ifdef USE_HOST
+  // Host uses slot count instead of byte size
+  this->log_buffer_ = esphome::make_unique<logger::TaskLogBufferHost>(total_buffer_size);
+#elif defined(USE_ESP32)
   this->log_buffer_ = esphome::make_unique<logger::TaskLogBuffer>(total_buffer_size);
+#elif defined(USE_LIBRETINY)
+  this->log_buffer_ = esphome::make_unique<logger::TaskLogBufferLibreTiny>(total_buffer_size);
+#endif
 
-  // Start with loop disabled when using task buffer (unless using USB CDC)
+#if defined(USE_ESP32) || defined(USE_LIBRETINY)
+  // Start with loop disabled when using task buffer (unless using USB CDC on ESP32)
   // The loop will be enabled automatically when messages arrive
   this->disable_loop_when_buffer_empty_();
+#endif
 }
 #endif
 
@@ -187,41 +235,48 @@ void Logger::process_messages_() {
 #ifdef USE_ESPHOME_TASK_LOG_BUFFER
   // Process any buffered messages when available
   if (this->log_buffer_->has_messages()) {
+#ifdef USE_HOST
+    logger::TaskLogBufferHost::LogMessage *message;
+    while (this->log_buffer_->get_message_main_loop(&message)) {
+      const char *thread_name = message->thread_name[0] != '\0' ? message->thread_name : nullptr;
+      this->format_buffered_message_and_notify_(message->level, message->tag, message->line, thread_name, message->text,
+                                                message->text_length);
+      this->log_buffer_->release_message_main_loop();
+      this->write_tx_buffer_to_console_();
+    }
+#elif defined(USE_ESP32)
     logger::TaskLogBuffer::LogMessage *message;
     const char *text;
     void *received_token;
-
-    // Process messages from the buffer
     while (this->log_buffer_->borrow_message_main_loop(&message, &text, &received_token)) {
-      this->tx_buffer_at_ = 0;
-      // Use the thread name that was stored when the message was created
-      // This avoids potential crashes if the task no longer exists
       const char *thread_name = message->thread_name[0] != '\0' ? message->thread_name : nullptr;
-      this->write_header_to_buffer_(message->level, message->tag, message->line, thread_name, this->tx_buffer_,
-                                    &this->tx_buffer_at_, this->tx_buffer_size_);
-      this->write_body_to_buffer_(text, message->text_length, this->tx_buffer_, &this->tx_buffer_at_,
-                                  this->tx_buffer_size_);
-      this->write_footer_to_buffer_(this->tx_buffer_, &this->tx_buffer_at_, this->tx_buffer_size_);
-      this->tx_buffer_[this->tx_buffer_at_] = '\0';
-      size_t msg_len = this->tx_buffer_at_;  // We already know the length from tx_buffer_at_
-      for (auto *listener : this->log_listeners_)
-        listener->on_log(message->level, message->tag, this->tx_buffer_, msg_len);
-      // At this point all the data we need from message has been transferred to the tx_buffer
-      // so we can release the message to allow other tasks to use it as soon as possible.
+      this->format_buffered_message_and_notify_(message->level, message->tag, message->line, thread_name, text,
+                                                message->text_length);
+      // Release the message to allow other tasks to use it as soon as possible
       this->log_buffer_->release_message_main_loop(received_token);
-
-      // Write to console from the main loop to prevent corruption from concurrent writes
-      // This ensures all log messages appear on the console in a clean, serialized manner
-      // Note: Messages may appear slightly out of order due to async processing, but
-      // this is preferred over corrupted/interleaved console output
       this->write_tx_buffer_to_console_();
     }
-  } else {
+#elif defined(USE_LIBRETINY)
+    logger::TaskLogBufferLibreTiny::LogMessage *message;
+    const char *text;
+    while (this->log_buffer_->borrow_message_main_loop(&message, &text)) {
+      const char *thread_name = message->thread_name[0] != '\0' ? message->thread_name : nullptr;
+      this->format_buffered_message_and_notify_(message->level, message->tag, message->line, thread_name, text,
+                                                message->text_length);
+      // Release the message to allow other tasks to use it as soon as possible
+      this->log_buffer_->release_message_main_loop();
+      this->write_tx_buffer_to_console_();
+    }
+#endif
+  }
+#if defined(USE_ESP32) || defined(USE_LIBRETINY)
+  else {
     // No messages to process, disable loop if appropriate
     // This reduces overhead when there's no async logging activity
     this->disable_loop_when_buffer_empty_();
   }
 #endif
+#endif  // USE_ESPHOME_TASK_LOG_BUFFER
 }
 
 void Logger::set_baud_rate(uint32_t baud_rate) { this->baud_rate_ = baud_rate; }
@@ -271,7 +326,11 @@ void Logger::dump_config() {
 #endif
 #ifdef USE_ESPHOME_TASK_LOG_BUFFER
   if (this->log_buffer_) {
-    ESP_LOGCONFIG(TAG, "  Task Log Buffer Size: %u", this->log_buffer_->size());
+#ifdef USE_HOST
+    ESP_LOGCONFIG(TAG, "  Task Log Buffer Slots: %u", static_cast<unsigned int>(this->log_buffer_->size()));
+#else
+    ESP_LOGCONFIG(TAG, "  Task Log Buffer Size: %u bytes", static_cast<unsigned int>(this->log_buffer_->size()));
+#endif
   }
 #endif
 
