@@ -9,18 +9,9 @@
 
 #include <mDNS.h>
 
-namespace esphome {
-namespace mdns {
-
-void MDNSComponent::setup() {
-#ifdef USE_MDNS_STORE_SERVICES
-  this->compile_records_(this->services_);
-  const auto &services = this->services_;
-#else
-  StaticVector<MDNSService, MDNS_SERVICE_COUNT> services;
-  this->compile_records_(services);
-#endif
+namespace esphome::mdns {
 
+static void register_libretiny(MDNSComponent *, StaticVector<MDNSService, MDNS_SERVICE_COUNT> &services) {
   MDNS.begin(App.get_name().c_str());
 
   for (const auto &service : services) {
@@ -44,9 +35,10 @@ void MDNSComponent::setup() {
   }
 }
 
+void MDNSComponent::setup() { this->setup_buffers_and_register_(register_libretiny); }
+
 void MDNSComponent::on_shutdown() {}
 
-}  // namespace mdns
-}  // namespace esphome
+}  // namespace esphome::mdns
 
 #endif
