@@ -8,8 +8,7 @@
 #include "esphome/components/fan/fan.h"
 #include "mqtt_component.h"
 
-namespace esphome {
-namespace mqtt {
+namespace esphome::mqtt {
 
 class MQTTFanComponent : public mqtt::MQTTComponent {
  public:
@@ -37,7 +36,7 @@ class MQTTFanComponent : public mqtt::MQTTComponent {
   bool send_initial_state() override;
   bool publish_state();
   /// 'fan' component type for discovery.
-  std::string component_type() const override;
+  const char *component_type() const override;
 
   fan::Fan *get_state() const;
 
@@ -47,8 +46,7 @@ class MQTTFanComponent : public mqtt::MQTTComponent {
   fan::Fan *state_;
 };
 
-}  // namespace mqtt
-}  // namespace esphome
+}  // namespace esphome::mqtt
 
 #endif
 #endif  // USE_MQTT
