@@ -8,8 +8,7 @@
 #include "esphome/components/update/update_entity.h"
 #include "mqtt_component.h"
 
-namespace esphome {
-namespace mqtt {
+namespace esphome::mqtt {
 
 class MQTTUpdateComponent : public mqtt::MQTTComponent {
  public:
@@ -28,14 +27,13 @@ class MQTTUpdateComponent : public mqtt::MQTTComponent {
 
  protected:
   /// "update" component type.
-  std::string component_type() const override;
+  const char *component_type() const override;
   const EntityBase *get_entity() const override;
 
   update::UpdateEntity *update_;
 };
 
-}  // namespace mqtt
-}  // namespace esphome
+}  // namespace esphome::mqtt
 
 #endif  // USE_UPDATE
 #endif  // USE_MQTT
