@@ -25,9 +25,13 @@ class PngDecoder : public ImageDecoder {
   int prepare(size_t download_size) override;
   int HOT decode(uint8_t *buffer, size_t size) override;
 
+  void increment_pixels_decoded(uint32_t count) { this->pixels_decoded_ += count; }
+  uint32_t get_pixels_decoded() const { return this->pixels_decoded_; }
+
  protected:
   RAMAllocator<pngle_t> allocator_;
   pngle_t *pngle_;
+  uint32_t pixels_decoded_{0};
 };
 
 }  // namespace online_image
