@@ -3,7 +3,7 @@
 namespace esphome {
 namespace ota {
 
-#ifdef USE_OTA_STATE_CALLBACK
+#ifdef USE_OTA_STATE_LISTENER
 OTAGlobalCallback *global_ota_callback{nullptr};  // NOLINT(cppcoreguidelines-avoid-non-const-global-variables)
 
 OTAGlobalCallback *get_global_ota_callback() {
@@ -13,7 +13,12 @@ OTAGlobalCallback *get_global_ota_callback() {
   return global_ota_callback;
 }
 
-void register_ota_platform(OTAComponent *ota_caller) { get_global_ota_callback()->register_ota(ota_caller); }
+void OTAComponent::notify_state_(OTAState state, float progress, uint8_t error) {
+  for (auto *listener : this->state_listeners_) {
+    listener->on_ota_state(state, progress, error);
+  }
+  get_global_ota_callback()->notify_ota_state(state, progress, error, this);
+}
 #endif
 
 }  // namespace ota
