@@ -21,9 +21,11 @@ void PCF8574Component::loop() {
   this->reset_pin_cache_();
 }
 void PCF8574Component::dump_config() {
-  ESP_LOGCONFIG(TAG, "PCF8574:");
+  ESP_LOGCONFIG(TAG,
+                "PCF8574:\n"
+                "  Is PCF8575: %s",
+                YESNO(this->pcf8575_));
   LOG_I2C_DEVICE(this)
-  ESP_LOGCONFIG(TAG, "  Is PCF8575: %s", YESNO(this->pcf8575_));
   if (this->is_failed()) {
     ESP_LOGE(TAG, ESP_LOG_MSG_COMM_FAIL);
   }
@@ -104,10 +106,8 @@ void PCF8574GPIOPin::setup() { pin_mode(flags_); }
 void PCF8574GPIOPin::pin_mode(gpio::Flags flags) { this->parent_->pin_mode(this->pin_, flags); }
 bool PCF8574GPIOPin::digital_read() { return this->parent_->digital_read(this->pin_) != this->inverted_; }
 void PCF8574GPIOPin::digital_write(bool value) { this->parent_->digital_write(this->pin_, value != this->inverted_); }
-std::string PCF8574GPIOPin::dump_summary() const {
-  char buffer[32];
-  snprintf(buffer, sizeof(buffer), "%u via PCF8574", pin_);
-  return buffer;
+size_t PCF8574GPIOPin::dump_summary(char *buffer, size_t len) const {
+  return snprintf(buffer, len, "%u via PCF8574", this->pin_);
 }
 
 }  // namespace pcf8574
