@@ -215,7 +215,8 @@ uint8_t PN7160::set_test_mode(const TestMode test_mode, const std::vector<uint8_
     result = rx.get_message();
     result.erase(result.begin(), result.begin() + 4);  // remove NCI header
     if (!result.empty()) {
-      ESP_LOGW(TAG, "Test results: %s", nfc::format_bytes(result).c_str());
+      char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+      ESP_LOGW(TAG, "Test results: %s", nfc::format_bytes_to(buf, result));
     }
   }
   return status;
@@ -246,7 +247,8 @@ uint8_t PN7160::reset_core_(const bool reset_config, const bool power) {
   }
 
   if (!rx.simple_status_response_is(nfc::STATUS_OK)) {
-    ESP_LOGE(TAG, "Invalid reset response: %s", nfc::format_bytes(rx.get_message()).c_str());
+    char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+    ESP_LOGE(TAG, "Invalid reset response: %s", nfc::format_bytes_to(buf, rx.get_message()));
     return rx.get_simple_status_response();
   }
   // read reset notification
@@ -258,15 +260,20 @@ uint8_t PN7160::reset_core_(const bool reset_config, const bool power) {
   if ((!rx.message_type_is(nfc::NCI_PKT_MT_CTRL_NOTIFICATION)) || (!rx.message_length_is(9)) ||
       (rx.get_message()[nfc::NCI_PKT_PAYLOAD_OFFSET] != 0x02) ||
       (rx.get_message()[nfc::NCI_PKT_PAYLOAD_OFFSET + 1] != (uint8_t) reset_config)) {
-    ESP_LOGE(TAG, "Reset notification was malformed: %s", nfc::format_bytes(rx.get_message()).c_str());
+    char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+    ESP_LOGE(TAG, "Reset notification was malformed: %s", nfc::format_bytes_to(buf, rx.get_message()));
     return nfc::STATUS_FAILED;
   }
 
-  ESP_LOGD(TAG, "Configuration %s", rx.get_message()[4] ? "reset" : "retained");
-  ESP_LOGD(TAG, "NCI version: %s", rx.get_message()[5] == 0x20 ? "2.0" : "1.0");
-  ESP_LOGD(TAG, "Manufacturer ID: 0x%02X", rx.get_message()[6]);
+  ESP_LOGD(TAG,
+           "Configuration %s\n"
+           "NCI version: %s\n"
+           "Manufacturer ID: 0x%02X",
+           rx.get_message()[4] ? "reset" : "retained", rx.get_message()[5] == 0x20 ? "2.0" : "1.0",
+           rx.get_message()[6]);
   rx.get_message().erase(rx.get_message().begin(), rx.get_message().begin() + 8);
-  ESP_LOGD(TAG, "Manufacturer info: %s", nfc::format_bytes(rx.get_message()).c_str());
+  char mfr_buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+  ESP_LOGD(TAG, "Manufacturer info: %s", nfc::format_bytes_to(mfr_buf, rx.get_message()));
 
   return nfc::STATUS_OK;
 }
@@ -281,7 +288,8 @@ uint8_t PN7160::init_core_() {
   }
 
   if (!rx.simple_status_response_is(nfc::STATUS_OK)) {
-    ESP_LOGE(TAG, "Invalid initialise response: %s", nfc::format_bytes(rx.get_message()).c_str());
+    char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+    ESP_LOGE(TAG, "Invalid initialise response: %s", nfc::format_bytes_to(buf, rx.get_message()));
     return nfc::STATUS_FAILED;
   }
 
@@ -291,11 +299,15 @@ uint8_t PN7160::init_core_() {
   uint8_t flash_minor_version = rx.get_message()[20 + rx.get_message()[8]];
   std::vector<uint8_t> features(rx.get_message().begin() + 4, rx.get_message().begin() + 8);
 
-  ESP_LOGD(TAG, "Hardware version: %u", hw_version);
-  ESP_LOGD(TAG, "ROM code version: %u", rom_code_version);
-  ESP_LOGD(TAG, "FLASH major version: %u", flash_major_version);
-  ESP_LOGD(TAG, "FLASH minor version: %u", flash_minor_version);
-  ESP_LOGD(TAG, "Features: %s", nfc::format_bytes(features).c_str());
+  char feat_buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+  ESP_LOGD(TAG,
+           "Hardware version: %u\n"
+           "ROM code version: %u\n"
+           "FLASH major version: %u\n"
+           "FLASH minor version: %u\n"
+           "Features: %s",
+           hw_version, rom_code_version, flash_major_version, flash_minor_version,
+           nfc::format_bytes_to(feat_buf, features));
 
   return rx.get_simple_status_response();
 }
@@ -594,7 +606,8 @@ void PN7160::erase_tag_(const uint8_t tag_index) {
     for (auto *listener : this->tag_listeners_) {
       listener->tag_off(*this->discovered_endpoint_[tag_index].tag);
     }
-    ESP_LOGI(TAG, "Tag %s removed", nfc::format_uid(this->discovered_endpoint_[tag_index].tag->get_uid()).c_str());
+    char uid_buf[nfc::FORMAT_UID_BUFFER_SIZE];
+    ESP_LOGI(TAG, "Tag %s removed", nfc::format_uid_to(uid_buf, this->discovered_endpoint_[tag_index].tag->get_uid()));
     this->discovered_endpoint_.erase(this->discovered_endpoint_.begin() + tag_index);
   }
 }
@@ -791,26 +804,33 @@ void PN7160::process_message_() {
             ESP_LOGV(TAG, "Unimplemented NCI Core OID received: 0x%02X", rx.get_oid());
         }
       } else {
-        ESP_LOGV(TAG, "Unimplemented notification: %s", nfc::format_bytes(rx.get_message()).c_str());
+        char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+        ESP_LOGV(TAG, "Unimplemented notification: %s", nfc::format_bytes_to(buf, rx.get_message()));
       }
       break;
 
-    case nfc::NCI_PKT_MT_CTRL_RESPONSE:
+    case nfc::NCI_PKT_MT_CTRL_RESPONSE: {
+      char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
       ESP_LOGV(TAG, "Unimplemented GID: 0x%02X  OID: 0x%02X  Full response: %s", rx.get_gid(), rx.get_oid(),
-               nfc::format_bytes(rx.get_message()).c_str());
+               nfc::format_bytes_to(buf, rx.get_message()));
       break;
+    }
 
-    case nfc::NCI_PKT_MT_CTRL_COMMAND:
-      ESP_LOGV(TAG, "Unimplemented command: %s", nfc::format_bytes(rx.get_message()).c_str());
+    case nfc::NCI_PKT_MT_CTRL_COMMAND: {
+      char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+      ESP_LOGV(TAG, "Unimplemented command: %s", nfc::format_bytes_to(buf, rx.get_message()));
       break;
+    }
 
     case nfc::NCI_PKT_MT_DATA:
       this->process_data_message_(rx);
       break;
 
-    default:
-      ESP_LOGV(TAG, "Unimplemented message type: %s", nfc::format_bytes(rx.get_message()).c_str());
+    default: {
+      char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+      ESP_LOGV(TAG, "Unimplemented message type: %s", nfc::format_bytes_to(buf, rx.get_message()));
       break;
+    }
   }
 }
 
@@ -871,8 +891,8 @@ void PN7160::process_rf_intf_activated_oid_(nfc::NciMessage &rx) {  // an endpoi
 
       case EP_WRITE:
         if (this->next_task_message_to_write_ != nullptr) {
-          ESP_LOGD(TAG, "  Tag writing");
-          ESP_LOGD(TAG, "  Tag formatting");
+          ESP_LOGD(TAG, "  Tag writing\n"
+                        "  Tag formatting");
           if (this->format_endpoint_(working_endpoint.tag->get_uid()) != nfc::STATUS_OK) {
             ESP_LOGE(TAG, "  Tag could not be formatted for writing");
           } else {
@@ -891,8 +911,9 @@ void PN7160::process_rf_intf_activated_oid_(nfc::NciMessage &rx) {  // an endpoi
       case EP_READ:
       default:
         if (!working_endpoint.trig_called) {
+          char uid_buf[nfc::FORMAT_UID_BUFFER_SIZE];
           ESP_LOGI(TAG, "Read tag type %s with UID %s", working_endpoint.tag->get_tag_type().c_str(),
-                   nfc::format_uid(working_endpoint.tag->get_uid()).c_str());
+                   nfc::format_uid_to(uid_buf, working_endpoint.tag->get_uid()));
           if (this->read_endpoint_data_(*working_endpoint.tag) != nfc::STATUS_OK) {
             ESP_LOGW(TAG, "  Unable to read NDEF record(s)");
           } else if (working_endpoint.tag->has_ndef_message()) {
@@ -983,7 +1004,8 @@ void PN7160::process_rf_deactivate_oid_(nfc::NciMessage &rx) {
 }
 
 void PN7160::process_data_message_(nfc::NciMessage &rx) {
-  ESP_LOGVV(TAG, "Received data message: %s", nfc::format_bytes(rx.get_message()).c_str());
+  char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+  ESP_LOGVV(TAG, "Received data message: %s", nfc::format_bytes_to(buf, rx.get_message()));
 
   std::vector<uint8_t> ndef_response;
   this->card_emu_t4t_get_response_(rx.get_message(), ndef_response);
@@ -997,7 +1019,7 @@ void PN7160::process_data_message_(nfc::NciMessage &rx) {
                                  uint8_t(ndef_response_size & 0x00FF)};
   tx_msg.insert(tx_msg.end(), ndef_response.begin(), ndef_response.end());
   nfc::NciMessage tx(tx_msg);
-  ESP_LOGVV(TAG, "Sending data message: %s", nfc::format_bytes(tx.get_message()).c_str());
+  ESP_LOGVV(TAG, "Sending data message: %s", nfc::format_bytes_to(buf, tx.get_message()));
   if (this->transceive_(tx, rx, NFCC_DEFAULT_TIMEOUT, false) != nfc::STATUS_OK) {
     ESP_LOGE(TAG, "Sending reply for card emulation failed");
   }
@@ -1050,7 +1072,8 @@ void PN7160::card_emu_t4t_get_response_(std::vector<uint8_t> &response, std::vec
       uint16_t offset = (response[nfc::NCI_PKT_HEADER_SIZE + 2] << 8) + response[nfc::NCI_PKT_HEADER_SIZE + 3];
       uint8_t length = response[nfc::NCI_PKT_HEADER_SIZE + 4];
 
-      ESP_LOGVV(TAG, "Encoded NDEF message: %s", nfc::format_bytes(ndef_message).c_str());
+      char ndef_buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+      ESP_LOGVV(TAG, "Encoded NDEF message: %s", nfc::format_bytes_to(ndef_buf, ndef_message));
 
       if (length <= (ndef_msg_size + offset + 2)) {
         if (offset == 0) {
@@ -1089,7 +1112,8 @@ void PN7160::card_emu_t4t_get_response_(std::vector<uint8_t> &response, std::vec
 
       ndef_msg_written.insert(ndef_msg_written.end(), response.begin() + nfc::NCI_PKT_HEADER_SIZE + 5,
                               response.begin() + nfc::NCI_PKT_HEADER_SIZE + 5 + length);
-      ESP_LOGD(TAG, "Received %u-byte NDEF message: %s", length, nfc::format_bytes(ndef_msg_written).c_str());
+      char write_buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
+      ESP_LOGD(TAG, "Received %u-byte NDEF message: %s", length, nfc::format_bytes_to(write_buf, ndef_msg_written));
       ndef_response.insert(ndef_response.end(), std::begin(CARD_EMU_T4T_OK), std::end(CARD_EMU_T4T_OK));
     }
   }
@@ -1098,6 +1122,7 @@ void PN7160::card_emu_t4t_get_response_(std::vector<uint8_t> &response, std::vec
 uint8_t PN7160::transceive_(nfc::NciMessage &tx, nfc::NciMessage &rx, const uint16_t timeout,
                             const bool expect_notification) {
   uint8_t retries = NFCC_MAX_COMM_FAILS;
+  char buf[nfc::FORMAT_BYTES_BUFFER_SIZE];
 
   while (retries) {
     // first, send the message we need to send
@@ -1105,7 +1130,7 @@ uint8_t PN7160::transceive_(nfc::NciMessage &tx, nfc::NciMessage &rx, const uint
       ESP_LOGE(TAG, "Error sending message");
       return nfc::STATUS_FAILED;
     }
-    ESP_LOGVV(TAG, "Wrote: %s", nfc::format_bytes(tx.get_message()).c_str());
+    ESP_LOGVV(TAG, "Wrote: %s", nfc::format_bytes_to(buf, tx.get_message()));
     // next, the NFCC should send back a response
     if (this->read_nfcc(rx, timeout) != nfc::STATUS_OK) {
       ESP_LOGW(TAG, "Error receiving message");
@@ -1117,24 +1142,24 @@ uint8_t PN7160::transceive_(nfc::NciMessage &tx, nfc::NciMessage &rx, const uint
       break;
     }
   }
-  ESP_LOGVV(TAG, "Read: %s", nfc::format_bytes(rx.get_message()).c_str());
+  ESP_LOGVV(TAG, "Read: %s", nfc::format_bytes_to(buf, rx.get_message()));
   // validate the response based on the message type that was sent (command vs. data)
   if (!tx.message_type_is(nfc::NCI_PKT_MT_DATA)) {
     // for commands, the GID and OID should match and the status should be OK
     if ((rx.get_gid() != tx.get_gid()) || (rx.get_oid()) != tx.get_oid()) {
-      ESP_LOGE(TAG, "Incorrect response to command: %s", nfc::format_bytes(rx.get_message()).c_str());
+      ESP_LOGE(TAG, "Incorrect response to command: %s", nfc::format_bytes_to(buf, rx.get_message()));
       return nfc::STATUS_FAILED;
     }
 
     if (!rx.simple_status_response_is(nfc::STATUS_OK)) {
-      ESP_LOGE(TAG, "Error in response to command: %s", nfc::format_bytes(rx.get_message()).c_str());
+      ESP_LOGE(TAG, "Error in response to command: %s", nfc::format_bytes_to(buf, rx.get_message()));
     }
     return rx.get_simple_status_response();
   } else {
     // when requesting data from the endpoint, the first response is from the NFCC; we must validate this, first
     if ((!rx.message_type_is(nfc::NCI_PKT_MT_CTRL_NOTIFICATION)) || (!rx.gid_is(nfc::NCI_CORE_GID)) ||
         (!rx.oid_is(nfc::NCI_CORE_CONN_CREDITS_OID)) || (!rx.message_length_is(3))) {
-      ESP_LOGE(TAG, "Incorrect response to data message: %s", nfc::format_bytes(rx.get_message()).c_str());
+      ESP_LOGE(TAG, "Incorrect response to data message: %s", nfc::format_bytes_to(buf, rx.get_message()));
       return nfc::STATUS_FAILED;
     }
 
@@ -1144,7 +1169,7 @@ uint8_t PN7160::transceive_(nfc::NciMessage &tx, nfc::NciMessage &rx, const uint
         ESP_LOGE(TAG, "Error receiving data from endpoint");
         return nfc::STATUS_FAILED;
       }
-      ESP_LOGVV(TAG, "Read: %s", nfc::format_bytes(rx.get_message()).c_str());
+      ESP_LOGVV(TAG, "Read: %s", nfc::format_bytes_to(buf, rx.get_message()));
     }
 
     return nfc::STATUS_OK;
