@@ -1,13 +1,8 @@
-import esphome.codegen as cg
-import esphome.config_validation as cv
 from esphome import pins
+import esphome.codegen as cg
 from esphome.components import sensor
-from esphome.const import (
-    CONF_PIN,
-    STATE_CLASS_MEASUREMENT,
-    UNIT_SECOND,
-    ICON_TIMER,
-)
+import esphome.config_validation as cv
+from esphome.const import CONF_PIN, ICON_TIMER, STATE_CLASS_MEASUREMENT, UNIT_SECOND
 
 pulse_width_ns = cg.esphome_ns.namespace("pulse_width")
 
