@@ -11,7 +11,7 @@ namespace remote_transmitter {
 template<typename... Ts> class DigitalWriteAction : public Action<Ts...>, public Parented<RemoteTransmitterComponent> {
  public:
   TEMPLATABLE_VALUE(bool, value)
-  void play(Ts... x) override { this->parent_->digital_write(this->value_.value(x...)); }
+  void play(const Ts &...x) override { this->parent_->digital_write(this->value_.value(x...)); }
 };
 
 }  // namespace remote_transmitter
