@@ -1,10 +1,7 @@
 import esphome.codegen as cg
-import esphome.config_validation as cv
 from esphome.components import switch
-from esphome.const import (
-    ENTITY_CATEGORY_CONFIG,
-    ICON_RESTART,
-)
+import esphome.config_validation as cv
+from esphome.const import ENTITY_CATEGORY_CONFIG, ICON_RESTART
 
 restart_ns = cg.esphome_ns.namespace("restart")
 RestartSwitch = restart_ns.class_("RestartSwitch", switch.Switch, cg.Component)
