@@ -49,8 +49,10 @@ void SHTCXComponent::setup() {
 }
 
 void SHTCXComponent::dump_config() {
-  ESP_LOGCONFIG(TAG, "SHTCx:");
-  ESP_LOGCONFIG(TAG, "  Model: %s (%04x)", to_string(this->type_), this->sensor_id_);
+  ESP_LOGCONFIG(TAG,
+                "SHTCx:\n"
+                "  Model: %s (%04x)",
+                to_string(this->type_), this->sensor_id_);
   LOG_I2C_DEVICE(this);
   if (this->is_failed()) {
     ESP_LOGE(TAG, ESP_LOG_MSG_COMM_FAIL);
