@@ -18,8 +18,7 @@
 #include <coredecls.h>  // For esp_schedule()
 #endif
 
-namespace esphome {
-namespace socket {
+namespace esphome::socket {
 
 #ifdef USE_ESP8266
 // Flag to signal socket activity - checked by socket_delay() to exit early
@@ -72,7 +71,7 @@ class LWIPRawImpl : public Socket {
     errno = EINVAL;
     return nullptr;
   }
-  int bind(const struct sockaddr *name, socklen_t addrlen) override {
+  int bind(const struct sockaddr *name, socklen_t addrlen) final {
     if (pcb_ == nullptr) {
       errno = EBADF;
       return -1;
@@ -136,7 +135,7 @@ class LWIPRawImpl : public Socket {
     }
     return 0;
   }
-  int close() override {
+  int close() final {
     if (pcb_ == nullptr) {
       errno = ECONNRESET;
       return -1;
@@ -153,7 +152,7 @@ class LWIPRawImpl : public Socket {
     pcb_ = nullptr;
     return 0;
   }
-  int shutdown(int how) override {
+  int shutdown(int how) final {
     if (pcb_ == nullptr) {
       errno = ECONNRESET;
       return -1;
@@ -179,7 +178,7 @@ class LWIPRawImpl : public Socket {
     return 0;
   }
 
-  int getpeername(struct sockaddr *name, socklen_t *addrlen) override {
+  int getpeername(struct sockaddr *name, socklen_t *addrlen) final {
     if (pcb_ == nullptr) {
       errno = ECONNRESET;
       return -1;
@@ -190,14 +189,7 @@ class LWIPRawImpl : public Socket {
     }
     return this->ip2sockaddr_(&pcb_->remote_ip, pcb_->remote_port, name, addrlen);
   }
-  std::string getpeername() override {
-    if (pcb_ == nullptr) {
-      errno = ECONNRESET;
-      return "";
-    }
-    return this->format_ip_address_(pcb_->remote_ip);
-  }
-  int getsockname(struct sockaddr *name, socklen_t *addrlen) override {
+  int getsockname(struct sockaddr *name, socklen_t *addrlen) final {
     if (pcb_ == nullptr) {
       errno = ECONNRESET;
       return -1;
@@ -208,14 +200,7 @@ class LWIPRawImpl : public Socket {
     }
     return this->ip2sockaddr_(&pcb_->local_ip, pcb_->local_port, name, addrlen);
   }
-  std::string getsockname() override {
-    if (pcb_ == nullptr) {
-      errno = ECONNRESET;
-      return "";
-    }
-    return this->format_ip_address_(pcb_->local_ip);
-  }
-  int getsockopt(int level, int optname, void *optval, socklen_t *optlen) override {
+  int getsockopt(int level, int optname, void *optval, socklen_t *optlen) final {
     if (pcb_ == nullptr) {
       errno = ECONNRESET;
       return -1;
@@ -249,7 +234,7 @@ class LWIPRawImpl : public Socket {
     errno = EINVAL;
     return -1;
   }
-  int setsockopt(int level, int optname, const void *optval, socklen_t optlen) override {
+  int setsockopt(int level, int optname, const void *optval, socklen_t optlen) final {
     if (pcb_ == nullptr) {
       errno = ECONNRESET;
       return -1;
@@ -283,7 +268,7 @@ class LWIPRawImpl : public Socket {
     errno = EOPNOTSUPP;
     return -1;
   }
-  ssize_t read(void *buf, size_t len) override {
+  ssize_t read(void *buf, size_t len) final {
     if (pcb_ == nullptr) {
       errno = ECONNRESET;
       return -1;
@@ -341,7 +326,7 @@ class LWIPRawImpl : public Socket {
 
     return read;
   }
-  ssize_t readv(const struct iovec *iov, int iovcnt) override {
+  ssize_t readv(const struct iovec *iov, int iovcnt) final {
     ssize_t ret = 0;
     for (int i = 0; i < iovcnt; i++) {
       ssize_t err = read(reinterpret_cast<uint8_t *>(iov[i].iov_base), iov[i].iov_len);
@@ -359,7 +344,7 @@ class LWIPRawImpl : public Socket {
     return ret;
   }
 
-  ssize_t recvfrom(void *buf, size_t len, sockaddr *addr, socklen_t *addr_len) override {
+  ssize_t recvfrom(void *buf, size_t len, sockaddr *addr, socklen_t *addr_len) final {
     errno = ENOTSUP;
     return -1;
   }
@@ -413,7 +398,7 @@ class LWIPRawImpl : public Socket {
     }
     return 0;
   }
-  ssize_t write(const void *buf, size_t len) override {
+  ssize_t write(const void *buf, size_t len) final {
     ssize_t written = internal_write(buf, len);
     if (written == -1)
       return -1;
@@ -428,7 +413,7 @@ class LWIPRawImpl : public Socket {
     }
     return written;
   }
-  ssize_t writev(const struct iovec *iov, int iovcnt) override {
+  ssize_t writev(const struct iovec *iov, int iovcnt) final {
     ssize_t written = 0;
     for (int i = 0; i < iovcnt; i++) {
       ssize_t err = internal_write(reinterpret_cast<uint8_t *>(iov[i].iov_base), iov[i].iov_len);
@@ -454,12 +439,12 @@ class LWIPRawImpl : public Socket {
     }
     return written;
   }
-  ssize_t sendto(const void *buf, size_t len, int flags, const struct sockaddr *to, socklen_t tolen) override {
+  ssize_t sendto(const void *buf, size_t len, int flags, const struct sockaddr *to, socklen_t tolen) final {
     // return ::sendto(fd_, buf, len, flags, to, tolen);
     errno = ENOSYS;
     return -1;
   }
-  int setblocking(bool blocking) override {
+  int setblocking(bool blocking) final {
     if (pcb_ == nullptr) {
       errno = ECONNRESET;
       return -1;
@@ -518,19 +503,6 @@ class LWIPRawImpl : public Socket {
   }
 
  protected:
-  std::string format_ip_address_(const ip_addr_t &ip) {
-    char buffer[50] = {};
-    if (IP_IS_V4_VAL(ip)) {
-      inet_ntoa_r(ip, buffer, sizeof(buffer));
-    }
-#if LWIP_IPV6
-    else if (IP_IS_V6_VAL(ip)) {
-      inet6_ntoa_r(ip, buffer, sizeof(buffer));
-    }
-#endif
-    return std::string(buffer);
-  }
-
   int ip2sockaddr_(ip_addr_t *ip, uint16_t port, struct sockaddr *name, socklen_t *addrlen) {
     if (family_ == AF_INET) {
       if (*addrlen < sizeof(struct sockaddr_in)) {
@@ -585,7 +557,7 @@ class LWIPRawImpl : public Socket {
 
 // Listening socket class - only allocates accept queue when needed (for bind+listen sockets)
 // This saves 16 bytes (12 bytes array + 1 byte count + 3 bytes padding) for regular connected sockets on ESP8266/RP2040
-class LWIPRawListenImpl : public LWIPRawImpl {
+class LWIPRawListenImpl final : public LWIPRawImpl {
  public:
   LWIPRawListenImpl(sa_family_t family, struct tcp_pcb *pcb) : LWIPRawImpl(family, pcb) {}
 
@@ -711,7 +683,6 @@ std::unique_ptr<Socket> socket_loop_monitored(int domain, int type, int protocol
   return socket(domain, type, protocol);
 }
 
-}  // namespace socket
-}  // namespace esphome
+}  // namespace esphome::socket
 
 #endif  // USE_SOCKET_IMPL_LWIP_TCP
