@@ -47,14 +47,15 @@ void SunGTIL2::loop() {
   }
 }
 
-std::string SunGTIL2::state_to_string_(uint8_t state) {
+const char *SunGTIL2::state_to_string_(uint8_t state, std::span<char, STATE_BUFFER_SIZE> buffer) {
   switch (state) {
     case 0x02:
       return "Starting voltage too low";
     case 0x07:
       return "Working";
     default:
-      return str_sprintf("Unknown (0x%02x)", state);
+      snprintf(buffer.data(), buffer.size(), "Unknown (0x%02x)", state);
+      return buffer.data();
   }
 }
 
@@ -106,12 +107,11 @@ void SunGTIL2::handle_char_(uint8_t c) {
 #endif
 #ifdef USE_TEXT_SENSOR
   if (this->state_ != nullptr) {
-    this->state_->publish_state(this->state_to_string_(msg.state));
+    char state_buffer[STATE_BUFFER_SIZE];
+    this->state_->publish_state(this->state_to_string_(msg.state, state_buffer));
   }
   if (this->serial_number_ != nullptr) {
-    std::string serial_number;
-    serial_number.assign(msg.serial_number, 10);
-    this->serial_number_->publish_state(serial_number);
+    this->serial_number_->publish_state(msg.serial_number, 10);
   }
 #endif
 }
