@@ -21,12 +21,14 @@ void TLC5947::setup() {
   this->pwm_amounts_.resize(this->num_chips_ * N_CHANNELS_PER_CHIP, 0);
 }
 void TLC5947::dump_config() {
-  ESP_LOGCONFIG(TAG, "TLC5947:");
+  ESP_LOGCONFIG(TAG,
+                "TLC5947:\n"
+                "  Number of chips: %u",
+                this->num_chips_);
   LOG_PIN("  Data Pin: ", this->data_pin_);
   LOG_PIN("  Clock Pin: ", this->clock_pin_);
   LOG_PIN("  LAT Pin: ", this->lat_pin_);
   LOG_PIN("  OE Pin: ", this->outenable_pin_);
-  ESP_LOGCONFIG(TAG, "  Number of chips: %u", this->num_chips_);
 }
 
 void TLC5947::loop() {
