@@ -1,3 +1,5 @@
+import logging
+
 from esphome import pins
 import esphome.codegen as cg
 from esphome.components import sensor
@@ -11,6 +13,8 @@ from esphome.const import (
     UNIT_METER,
 )
 
+_LOGGER = logging.getLogger(__name__)
+
 CONF_PULSE_TIME = "pulse_time"
 
 ultrasonic_ns = cg.esphome_ns.namespace("ultrasonic")
@@ -28,9 +32,9 @@ CONFIG_SCHEMA = (
     )
     .extend(
         {
-            cv.Required(CONF_TRIGGER_PIN): pins.gpio_output_pin_schema,
+            cv.Required(CONF_TRIGGER_PIN): pins.internal_gpio_output_pin_schema,
             cv.Required(CONF_ECHO_PIN): pins.internal_gpio_input_pin_schema,
-            cv.Optional(CONF_TIMEOUT, default="2m"): cv.distance,
+            cv.Optional(CONF_TIMEOUT): cv.distance,
             cv.Optional(
                 CONF_PULSE_TIME, default="10us"
             ): cv.positive_time_period_microseconds,
@@ -49,5 +53,11 @@ async def to_code(config):
     echo = await cg.gpio_pin_expression(config[CONF_ECHO_PIN])
     cg.add(var.set_echo_pin(echo))
 
-    cg.add(var.set_timeout_us(config[CONF_TIMEOUT] / (0.000343 / 2)))
+    # Remove before 2026.8.0
+    if CONF_TIMEOUT in config:
+        _LOGGER.warning(
+            "'timeout' option is deprecated and will be removed in 2026.8.0. "
+            "The option has no effect and can be safely removed."
+        )
+
     cg.add(var.set_pulse_time_us(config[CONF_PULSE_TIME]))
