@@ -71,7 +71,7 @@ bool parse_xiaomi_value(uint16_t value_type, const uint8_t *data, uint8_t value_
 bool parse_xiaomi_message(const std::vector<uint8_t> &message, XiaomiParseResult &result);
 optional<XiaomiParseResult> parse_xiaomi_header(const esp32_ble_tracker::ServiceData &service_data);
 bool decrypt_xiaomi_payload(std::vector<uint8_t> &raw, const uint8_t *bindkey, const uint64_t &address);
-bool report_xiaomi_results(const optional<XiaomiParseResult> &result, const std::string &address);
+bool report_xiaomi_results(const optional<XiaomiParseResult> &result, const char *address);
 
 class XiaomiListener : public esp32_ble_tracker::ESPBTDeviceListener {
  public:
