@@ -50,25 +50,7 @@ void ZephyrGPIOPin::attach_interrupt(void (*func)(void *), void *arg, gpio::Inte
 }
 
 void ZephyrGPIOPin::setup() {
-  const struct device *gpio = nullptr;
-  if (this->pin_ < 32) {
-#define GPIO0 DT_NODELABEL(gpio0)
-#if DT_NODE_HAS_STATUS(GPIO0, okay)
-    gpio = DEVICE_DT_GET(GPIO0);
-#else
-#error "gpio0 is disabled"
-#endif
-  } else {
-#define GPIO1 DT_NODELABEL(gpio1)
-#if DT_NODE_HAS_STATUS(GPIO1, okay)
-    gpio = DEVICE_DT_GET(GPIO1);
-#else
-#error "gpio1 is disabled"
-#endif
-  }
-  if (device_is_ready(gpio)) {
-    this->gpio_ = gpio;
-  } else {
+  if (!device_is_ready(this->gpio_)) {
     ESP_LOGE(TAG, "gpio %u is not ready.", this->pin_);
     return;
   }
@@ -79,23 +61,22 @@ void ZephyrGPIOPin::pin_mode(gpio::Flags flags) {
   if (nullptr == this->gpio_) {
     return;
   }
-  auto ret = gpio_pin_configure(this->gpio_, this->pin_ % 32, flags_to_mode(flags, this->inverted_, this->value_));
+  auto ret = gpio_pin_configure(this->gpio_, this->pin_ % this->gpio_size_,
+                                flags_to_mode(flags, this->inverted_, this->value_));
   if (ret != 0) {
     ESP_LOGE(TAG, "gpio %u cannot be configured %d.", this->pin_, ret);
   }
 }
 
-std::string ZephyrGPIOPin::dump_summary() const {
-  char buffer[32];
-  snprintf(buffer, sizeof(buffer), "GPIO%u, P%u.%u", this->pin_, this->pin_ / 32, this->pin_ % 32);
-  return buffer;
+size_t ZephyrGPIOPin::dump_summary(char *buffer, size_t len) const {
+  return snprintf(buffer, len, "GPIO%u, %s%u", this->pin_, this->pin_name_prefix_, this->pin_ % this->gpio_size_);
 }
 
 bool ZephyrGPIOPin::digital_read() {
   if (nullptr == this->gpio_) {
     return false;
   }
-  return bool(gpio_pin_get(this->gpio_, this->pin_ % 32) != this->inverted_);
+  return bool(gpio_pin_get(this->gpio_, this->pin_ % this->gpio_size_) != this->inverted_);
 }
 
 void ZephyrGPIOPin::digital_write(bool value) {
@@ -105,7 +86,7 @@ void ZephyrGPIOPin::digital_write(bool value) {
   if (nullptr == this->gpio_) {
     return;
   }
-  gpio_pin_set(this->gpio_, this->pin_ % 32, value != this->inverted_ ? 1 : 0);
+  gpio_pin_set(this->gpio_, this->pin_ % this->gpio_size_, value != this->inverted_ ? 1 : 0);
 }
 void ZephyrGPIOPin::detach_interrupt() const {
   // TODO
