@@ -4,7 +4,7 @@ from enum import Enum
 
 from esphome.enum import StrEnum
 
-__version__ = "2025.12.5"
+__version__ = "2026.1.2"
 
 ALLOWED_NAME_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789-_"
 VALID_SUBSTITUTIONS_CHARACTERS = (
@@ -123,6 +123,7 @@ CONF_ADDRESS = "address"
 CONF_ADDRESSABLE_LIGHT_ID = "addressable_light_id"
 CONF_ADVANCED = "advanced"
 CONF_AFTER = "after"
+CONF_ALGORITHM_TUNING = "algorithm_tuning"
 CONF_ALL = "all"
 CONF_ALLOW_OTHER_USES = "allow_other_uses"
 CONF_ALPHA = "alpha"
@@ -246,6 +247,7 @@ CONF_COMPENSATION = "compensation"
 CONF_COMPILE_PROCESS_LIMIT = "compile_process_limit"
 CONF_COMPONENT_ID = "component_id"
 CONF_COMPONENTS = "components"
+CONF_COMPRESSION = "compression"
 CONF_CONDITION = "condition"
 CONF_CONDITION_ID = "condition_id"
 CONF_CONDUCTIVITY = "conductivity"
@@ -435,6 +437,7 @@ CONF_GAIN_FACTOR = "gain_factor"
 CONF_GAMMA_CORRECT = "gamma_correct"
 CONF_GAS_RESISTANCE = "gas_resistance"
 CONF_GATEWAY = "gateway"
+CONF_GATING_MAX_DURATION_MINUTES = "gating_max_duration_minutes"
 CONF_GLASS_ATTENUATION_FACTOR = "glass_attenuation_factor"
 CONF_GLYPHS = "glyphs"
 CONF_GPIO = "gpio"
@@ -497,6 +500,7 @@ CONF_INCLUDE_INTERNAL = "include_internal"
 CONF_INCLUDES = "includes"
 CONF_INCLUDES_C = "includes_c"
 CONF_INDEX = "index"
+CONF_INDEX_OFFSET = "index_offset"
 CONF_INDOOR = "indoor"
 CONF_INFRARED = "infrared"
 CONF_INIT_SEQUENCE = "init_sequence"
@@ -534,6 +538,8 @@ CONF_LAMBDA = "lambda"
 CONF_LAST_CONFIDENCE = "last_confidence"
 CONF_LAST_FINGER_ID = "last_finger_id"
 CONF_LATITUDE = "latitude"
+CONF_LEARNING_TIME_GAIN_HOURS = "learning_time_gain_hours"
+CONF_LEARNING_TIME_OFFSET_HOURS = "learning_time_offset_hours"
 CONF_LED = "led"
 CONF_LEGEND = "legend"
 CONF_LENGTH = "length"
@@ -645,7 +651,9 @@ CONF_NEVER = "never"
 CONF_NEW_PASSWORD = "new_password"
 CONF_NITROGEN_DIOXIDE = "nitrogen_dioxide"
 CONF_NOISE_LEVEL = "noise_level"
+CONF_NORMALIZED_OFFSET_SLOPE = "normalized_offset_slope"
 CONF_NOTIFY = "notify"
+CONF_NOX = "nox"
 CONF_NUM_ATTEMPTS = "num_attempts"
 CONF_NUM_CHANNELS = "num_channels"
 CONF_NUM_CHIPS = "num_chips"
@@ -672,6 +680,7 @@ CONF_ON_CLIENT_CONNECTED = "on_client_connected"
 CONF_ON_CLIENT_DISCONNECTED = "on_client_disconnected"
 CONF_ON_CONNECT = "on_connect"
 CONF_ON_CONTROL = "on_control"
+CONF_ON_DATA = "on_data"
 CONF_ON_DIRECTION_SET = "on_direction_set"
 CONF_ON_DISCONNECT = "on_disconnect"
 CONF_ON_DOUBLE_CLICK = "on_double_click"
@@ -702,6 +711,7 @@ CONF_ON_RELEASE = "on_release"
 CONF_ON_RESPONSE = "on_response"
 CONF_ON_SHUTDOWN = "on_shutdown"
 CONF_ON_SPEED_SET = "on_speed_set"
+CONF_ON_START = "on_start"
 CONF_ON_STATE = "on_state"
 CONF_ON_SUCCESS = "on_success"
 CONF_ON_TAG = "on_tag"
@@ -939,6 +949,7 @@ CONF_STATE_TOPIC = "state_topic"
 CONF_STATIC_IP = "static_ip"
 CONF_STATUS = "status"
 CONF_STB_PIN = "stb_pin"
+CONF_STD_INITIAL = "std_initial"
 CONF_STEP = "step"
 CONF_STEP_DELAY = "step_delay"
 CONF_STEP_MODE = "step_mode"
@@ -1006,6 +1017,7 @@ CONF_TILT_COMMAND_TOPIC = "tilt_command_topic"
 CONF_TILT_LAMBDA = "tilt_lambda"
 CONF_TILT_STATE_TOPIC = "tilt_state_topic"
 CONF_TIME = "time"
+CONF_TIME_CONSTANT = "time_constant"
 CONF_TIME_ID = "time_id"
 CONF_TIMEOUT = "timeout"
 CONF_TIMES = "times"
@@ -1060,6 +1072,8 @@ CONF_VERSION = "version"
 CONF_VIBRATIONS = "vibrations"
 CONF_VISIBLE = "visible"
 CONF_VISUAL = "visual"
+CONF_VOC = "voc"
+CONF_VOC_BASELINE = "voc_baseline"
 CONF_VOLTAGE = "voltage"
 CONF_VOLTAGE_ATTENUATION = "voltage_attenuation"
 CONF_VOLTAGE_DIVIDER = "voltage_divider"
@@ -1074,6 +1088,7 @@ CONF_WARM_WHITE = "warm_white"
 CONF_WARM_WHITE_COLOR_TEMPERATURE = "warm_white_color_temperature"
 CONF_WATCHDOG_THRESHOLD = "watchdog_threshold"
 CONF_WATCHDOG_TIMEOUT = "watchdog_timeout"
+CONF_WATER_HEATER = "water_heater"
 CONF_WEB_SERVER = "web_server"
 CONF_WEB_SERVER_ID = "web_server_id"
 CONF_WEIGHT = "weight"
@@ -1167,6 +1182,7 @@ ICON_TIMELAPSE = "mdi:timelapse"
 ICON_TIMER = "mdi:timer-outline"
 ICON_VIBRATE = "mdi:vibrate"
 ICON_WATER = "mdi:water"
+ICON_WATER_HEATER = "mdi:water-boiler"
 ICON_WATER_PERCENT = "mdi:water-percent"
 ICON_WEATHER_SUNSET = "mdi:weather-sunset"
 ICON_WEATHER_SUNSET_DOWN = "mdi:weather-sunset-down"
