@@ -8,9 +8,15 @@
 // ESP8266 uses Arduino macros
 #define ESPHOME_F(string_literal) F(string_literal)
 #define ESPHOME_PGM_P PGM_P
+#define ESPHOME_PSTR(s) PSTR(s)
 #define ESPHOME_strncpy_P strncpy_P
+#define ESPHOME_strncat_P strncat_P
+#define ESPHOME_snprintf_P snprintf_P
 #else
 #define ESPHOME_F(string_literal) (string_literal)
 #define ESPHOME_PGM_P const char *
+#define ESPHOME_PSTR(s) (s)
 #define ESPHOME_strncpy_P strncpy
+#define ESPHOME_strncat_P strncat
+#define ESPHOME_snprintf_P snprintf
 #endif
