@@ -3,6 +3,7 @@
 #include <cstdint>
 #include <cstdlib>
 #include <ctime>
+#include <span>
 #include <string>
 
 namespace esphome {
@@ -13,6 +14,9 @@ uint8_t days_in_month(uint8_t month, uint16_t year);
 
 /// A more user-friendly version of struct tm from time.h
 struct ESPTime {
+  /// Buffer size required for strftime output
+  static constexpr size_t STRFTIME_BUFFER_SIZE = 128;
+
   /** seconds after the minute [0-60]
    * @note second is generally 0-59; the extra range is to accommodate leap seconds.
    */
@@ -43,14 +47,22 @@ struct ESPTime {
    */
   size_t strftime(char *buffer, size_t buffer_len, const char *format);
 
+  /** Format time into a fixed-size buffer, returns length written.
+   *
+   * This is the preferred method for avoiding heap allocations. The buffer size is enforced at compile-time.
+   * On format error, writes "ERROR" to the buffer and returns 5.
+   * @see https://www.gnu.org/software/libc/manual/html_node/Formatting-Calendar-Time.html#index-strftime
+   */
+  size_t strftime_to(std::span<char, STRFTIME_BUFFER_SIZE> buffer, const char *format);
+
   /** Convert this ESPTime struct to a string as specified by the format argument.
    * @see https://en.cppreference.com/w/c/chrono/strftime
    *
    * @warning This method returns a dynamically allocated string which can cause heap fragmentation with some
-   * microcontrollers.
+   * microcontrollers. Prefer strftime_to() for heap-free formatting.
    *
    * @warning This method can return "ERROR" when the underlying strftime() call fails or when the
-   * output exceeds 128 bytes.
+   * output exceeds STRFTIME_BUFFER_SIZE bytes.
    */
   std::string strftime(const std::string &format);
 
