@@ -1,11 +1,13 @@
 from collections.abc import Callable
 import importlib
+import json
 import logging
 import os
 from pathlib import Path
 import re
 import shutil
 import stat
+import time
 from types import TracebackType
 
 from esphome import loader
@@ -19,10 +21,12 @@ from esphome.const import (
 from esphome.core import CORE, EsphomeError
 from esphome.helpers import (
     copy_file_if_changed,
+    cpp_string_escape,
     get_str_env,
     is_ha_addon,
     read_file,
     walk_files,
+    write_file,
     write_file_if_changed,
 )
 from esphome.storage_json import StorageJSON, storage_path
@@ -170,6 +174,7 @@ VERSION_H_FORMAT = """\
 """
 DEFINES_H_TARGET = "esphome/core/defines.h"
 VERSION_H_TARGET = "esphome/core/version.h"
+BUILD_INFO_DATA_H_TARGET = "esphome/core/build_info_data.h"
 ESPHOME_README_TXT = """
 THIS DIRECTORY IS AUTO-GENERATED, DO NOT MODIFY
 
@@ -203,10 +208,16 @@ def copy_src_tree():
     include_s = "\n".join(include_l)
 
     source_files_copy = source_files_map.copy()
-    ignore_targets = [Path(x) for x in (DEFINES_H_TARGET, VERSION_H_TARGET)]
+    ignore_targets = [
+        Path(x) for x in (DEFINES_H_TARGET, VERSION_H_TARGET, BUILD_INFO_DATA_H_TARGET)
+    ]
     for t in ignore_targets:
-        source_files_copy.pop(t)
+        source_files_copy.pop(t, None)
 
+    # Files to exclude from sources_changed tracking (generated files)
+    generated_files = {Path("esphome/core/build_info_data.h")}
+
+    sources_changed = False
     for fname in walk_files(CORE.relative_src_path("esphome")):
         p = Path(fname)
         if p.suffix not in SOURCE_FILE_EXTENSIONS:
@@ -220,28 +231,82 @@ def copy_src_tree():
         if target not in source_files_copy:
             # Source file removed, delete target
             p.unlink()
+            if target not in generated_files:
+                sources_changed = True
         else:
             src_file = source_files_copy.pop(target)
             with src_file.path() as src_path:
-                copy_file_if_changed(src_path, p)
+                if copy_file_if_changed(src_path, p) and target not in generated_files:
+                    sources_changed = True
 
     # Now copy new files
     for target, src_file in source_files_copy.items():
         dst_path = CORE.relative_src_path(*target.parts)
         with src_file.path() as src_path:
-            copy_file_if_changed(src_path, dst_path)
+            if (
+                copy_file_if_changed(src_path, dst_path)
+                and target not in generated_files
+            ):
+                sources_changed = True
 
     # Finally copy defines
-    write_file_if_changed(
+    if write_file_if_changed(
         CORE.relative_src_path("esphome", "core", "defines.h"), generate_defines_h()
-    )
+    ):
+        sources_changed = True
     write_file_if_changed(CORE.relative_build_path("README.txt"), ESPHOME_README_TXT)
-    write_file_if_changed(
+    if write_file_if_changed(
         CORE.relative_src_path("esphome.h"), ESPHOME_H_FORMAT.format(include_s)
-    )
-    write_file_if_changed(
+    ):
+        sources_changed = True
+    if write_file_if_changed(
         CORE.relative_src_path("esphome", "core", "version.h"), generate_version_h()
+    ):
+        sources_changed = True
+
+    # Generate new build_info files if needed
+    build_info_data_h_path = CORE.relative_src_path(
+        "esphome", "core", "build_info_data.h"
     )
+    build_info_json_path = CORE.relative_build_path("build_info.json")
+    config_hash, build_time, build_time_str, comment = get_build_info()
+
+    # Defensively force a rebuild if the build_info files don't exist, or if
+    # there was a config change which didn't actually cause a source change
+    if not build_info_data_h_path.exists():
+        sources_changed = True
+    else:
+        try:
+            existing = json.loads(build_info_json_path.read_text(encoding="utf-8"))
+            if (
+                existing.get("config_hash") != config_hash
+                or existing.get("esphome_version") != __version__
+            ):
+                sources_changed = True
+        except (json.JSONDecodeError, KeyError, OSError):
+            sources_changed = True
+
+    # Write build_info header and JSON metadata
+    if sources_changed:
+        write_file(
+            build_info_data_h_path,
+            generate_build_info_data_h(
+                config_hash, build_time, build_time_str, comment
+            ),
+        )
+        write_file(
+            build_info_json_path,
+            json.dumps(
+                {
+                    "config_hash": config_hash,
+                    "build_time": build_time,
+                    "build_time_str": build_time_str,
+                    "esphome_version": __version__,
+                },
+                indent=2,
+            )
+            + "\n",
+        )
 
     platform = "esphome.components." + CORE.target_platform
     try:
@@ -267,6 +332,43 @@ def generate_version_h():
     )
 
 
+def get_build_info() -> tuple[int, int, str, str]:
+    """Calculate build_info values from current config.
+
+    Returns:
+        Tuple of (config_hash, build_time, build_time_str, comment)
+    """
+    config_hash = CORE.config_hash
+    build_time = int(time.time())
+    build_time_str = time.strftime("%Y-%m-%d %H:%M:%S %z", time.localtime(build_time))
+    comment = CORE.comment or ""
+    return config_hash, build_time, build_time_str, comment
+
+
+def generate_build_info_data_h(
+    config_hash: int, build_time: int, build_time_str: str, comment: str
+) -> str:
+    """Generate build_info_data.h header with config hash, build time, and comment."""
+    # cpp_string_escape returns '"escaped"', slice off the quotes since template has them
+    escaped_comment = cpp_string_escape(comment)[1:-1]
+    # +1 for null terminator
+    comment_size = len(comment) + 1
+    return f"""#pragma once
+// Auto-generated build_info data
+#define ESPHOME_CONFIG_HASH 0x{config_hash:08x}U  // NOLINT
+#define ESPHOME_BUILD_TIME {build_time}  // NOLINT
+#define ESPHOME_COMMENT_SIZE {comment_size}  // NOLINT
+#ifdef USE_ESP8266
+#include <pgmspace.h>
+static const char ESPHOME_BUILD_TIME_STR[] PROGMEM = "{build_time_str}";
+static const char ESPHOME_COMMENT_STR[] PROGMEM = "{escaped_comment}";
+#else
+static const char ESPHOME_BUILD_TIME_STR[] = "{build_time_str}";
+static const char ESPHOME_COMMENT_STR[] = "{escaped_comment}";
+#endif
+"""
+
+
 def write_cpp(code_s):
     path = CORE.relative_src_path("main.cpp")
     if path.is_file():
